from terminal_typing_test.utils import TIME_DP


class Monitor:
    def __init__(self, num_words_typed, start_time):
        self.num_words_typed = num_words_typed
        self.start_time = start_time
        self.end_time = None

    def print_results(self, scr, row):
        time_taken = self.end_time - self.start_time
        wpm = self.num_words_typed / time_taken * 60
        scr.addstr(row, 0, f"Time taken: {round(time_taken, TIME_DP)} seconds.")
        scr.addstr(row + 1, 0, f"Number of words typed: {self.num_words_typed}.")
        scr.addstr(row + 2, 0, f"Your WPM is {round(wpm, 1)}. Press Enter to quit.")



import curses
import time

TIME_DP = 1
ENTER_KEYS = {10, 13}

def print_char(scr, char, colour_profile):
    scr.attron(curses.color_pair(colour_profile))
    y, x = scr.getyx()
    scr.addstr(y, x, char)
    move_back(scr)
    scr.attroff(curses.color_pair(colour_profile))

def move_forward(scr):
    y, x = scr.getyx()
    h, w = scr.getmaxyx()
    if x < w - 1:
        scr.move(y, x + 1)
    else:
        scr.move(y + 1, 0)

def move_back(scr):
    y, x = scr.getyx()
    h, w = scr.getmaxyx()
    if x > 0:
        scr.move(y, x - 1)
    else:
        scr.move(y - 1, w - 1)


def wait_for_enter_pressed(scr):
    scr.nodelay(False)
    key = None
    while key not in ENTER_KEYS:
        key = scr.getch()
    return


def print_time(scr, row, start_time):
    y, x = scr.getyx()
    scr.addstr(row, 0, f"Time taken: {round(time.time() - start_time, TIME_DP)}s")
    scr.move(y, x)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['terminal_typing_test']

package_data = \
{'': ['*'], 'terminal_typing_test': ['text/*', 'text/short/*']}

entry_points = \
{'console_scripts': ['typingtest = terminal_typing_test.main:run']}

setup_kwargs = {
    'name': 'terminal-typing-test',
    'version': '0.1.2',
    'description': 'A terminal-based typing test.',
    'long_description': '# Terminal Typing Test\n\nFind your typing speed through the command line!\n\nInstall with `pip install terminal-typing-test`. Once installed, run `typingtest` from the command line.\n\n',
    'author': 'Fawaz Shah',
    'author_email': 'fawaz010@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import curses
import os
import random
import sys
import time

from terminal_typing_test.monitor import Monitor
from terminal_typing_test.utils import move_forward, move_back, print_time, print_char, wait_for_enter_pressed

ESCAPE = 27
BACKSPACE_KEYS = {8, 127}

CORRECT_COLOUR = 1
ERROR_COLOUR = 2
RESET_COLOUR = 3

text_directory = os.path.join(os.path.dirname(__file__), "text/short/")
file = random.choice(os.listdir(text_directory))
with open(text_directory + file) as f:
    TEST_STRING = f.read()
    TEST_STRING_LENGTH = len(TEST_STRING)


def main(stdscr):
    curses.use_default_colors()
    curses.init_pair(CORRECT_COLOUR, curses.COLOR_BLACK, curses.COLOR_GREEN)
    curses.init_pair(ERROR_COLOUR, curses.COLOR_BLACK, curses.COLOR_RED)
    curses.init_pair(RESET_COLOUR, -1, -1)
    stdscr.nodelay(True)

    monitor = Monitor(1, time.time())
    curr_char_idx = 0

    stdscr.addstr(0, 0, "Welcome to Typing Test! Press Ctrl-C to quit.")
    stdscr.addstr(4, 0, "Type the following:")
    stdscr.addstr(5, 0, TEST_STRING)
    stdscr.move(5, 0)

    while True:
        try:
            key = stdscr.getch()
            print_time(stdscr, 2, monitor.start_time)
        except KeyboardInterrupt:
            sys.exit()

        if key == curses.ERR:
            pass
        elif key == ESCAPE:
            break
        elif key in BACKSPACE_KEYS:
            if curr_char_idx > 0:
                print_char(stdscr, TEST_STRING[curr_char_idx], RESET_COLOUR)
                curr_char_idx -= 1
                move_back(stdscr)
        else:
            if curr_char_idx < TEST_STRING_LENGTH - 1:
                if TEST_STRING[curr_char_idx] == " ":
                    monitor.num_words_typed += 1
                colour_profile = CORRECT_COLOUR if key == ord(TEST_STRING[curr_char_idx]) else ERROR_COLOUR
                print_char(stdscr, TEST_STRING[curr_char_idx], colour_profile)
                curr_char_idx += 1
                move_forward(stdscr)
            else:
                monitor.end_time = time.time()
                stdscr.clear()
                monitor.print_results(stdscr, row=0)
                wait_for_enter_pressed(stdscr)
                break


def run():
    curses.wrapper(main)
